// write to a text file
// created by Zachary Seldess, 05/21/12
// Visualization Lab, King Abdullah University of Science and Technology
// Copyright  2012 King Abdullah University of Science and Technology. Some Rights Reserved.

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.io.*;
import com.cycling74.max.*;

public class writeText extends MaxObject {

	private String path;	
	private File filename;
	private FileWriter writer;
	private ArrayList<Object> al = new ArrayList<Object>();
	DecimalFormat df = new DecimalFormat("#0.000000");
	DateFormat datef = new SimpleDateFormat("yyyy-MM-dd_hh:mm:ss");  	
	
	writeText() {
		declareIO(1,1);
		setInletAssist(0, "messages to save text file");
		setOutletAssist(0, "bang upon save completion");
		createInfoOutlet(false);
		post("writeText by Zachary Seldess. Build May 21 2012.");
		post("Copyright  2012 King Abdullah University of Science and Technology. Some Rights Reserved."); 
	}

	public void write(Atom[] args) {
		if (args.length == 0) { // if no filename provided, generate one
			try {
				path = System.getProperty("user.dir"); // get working directory cross-platform				
				filename = new File(path + "writeText_" + getDateTime() + ".txt");
				writer = new FileWriter(filename, false);
				Object obj;
				for (int i = 0; i < al.size(); i++) {
					obj = al.get(i);
					if (obj.getClass().getName().equals("java.lang.String")) {
						writer.write((String)obj);
					}
					else if (obj.getClass().getName().equals("java.lang.Integer")) {
						writer.write(Integer.toString((Integer)obj));
					}
					else if (obj.getClass().getName().equals("java.lang.Float")) {
						//df.setMinimumFractionDigits(6);							
						writer.write(df.format((Float)obj));
					}
					//post("class: " + obj.getClass().getName());												
				}
				writer.flush();
				writer.close();
				outlet(0, "bang");				
			}
			catch (Exception e) {
				postError(e);
			}				
		}
		else {
			try {
				filename = new File(args[0].toString());					
				writer = new FileWriter(filename, false);
				Object obj;
				for (int i = 0; i < al.size(); i++) {
					obj = al.get(i);
					if (obj.getClass().getName().equals("java.lang.String")) {
						writer.write((String)obj);
					}
					else if (obj.getClass().getName().equals("java.lang.Integer")) {
						writer.write(Integer.toString((Integer)obj));
					}
					else if (obj.getClass().getName().equals("java.lang.Float")) {
						//df.setMinimumFractionDigits(6);							
						writer.write(df.format((Float)obj));
					}
					//post("class: " + obj.getClass().getName());
				}
				writer.flush();
				writer.close();
				outlet(0, "bang");
			}
			catch (Exception e) {
				postError(e);
			}
		}
	}
	
	// clear contents of ArrayList object
	public void clear() {
		al.clear();
	}
	
	// add newline to StringBuilder object
	public void cr() {
		String newline = System.getProperty("line.separator"); // ensures newline works cross-platform
		al.add(new String(newline));
	}
	
	// add tab to StringBuilder object
	public void tab() {
		al.add(new String("\t"));
	}
		
	public void inlet(int i) {
		try {
			al.add(new String(Integer.toString(i)));
			al.add(new String(" "));
		}
		catch (Exception e) {
			postError(e);
		}
	}
	
	public void inlet(float f) {
		try {
			al.add(new String(Float.toString(f)));
			al.add(new String(" "));
		}
		catch (Exception e) {
			postError(e);
		}	
	}
	
	// add list one element at a time to StringBuilder object
	public void list(Atom[] args) {			
		for (int i = 0; i < args.length; i++) {
			try {
				if (args[i].isString()) {			
					al.add(new String(args[i].getString()));
				}
				else if (args[i].isInt()) {
					al.add(new Integer(args[i].getInt()));
				}
				else if (args[i].isFloat()) {
					al.add(new Float(args[i].getFloat()));
				}				
				al.add(new String(" "));
			}
			catch (Exception e) {
				postError(e);
			}
		}
		//post("al:" + al);	
	}
	
	// add message one element at a time to StringBuilder object
	public void anything(String msg, Atom[] args) {	
		try {
			al.add(new String(msg));
			al.add(new String(" "));
		}
		catch (Exception e) {
			postError(e);
		}		
		for (int i = 0; i < args.length; i++) {
			try {
				if (args[i].isString()) {			
					al.add(new String(args[i].getString()));
				}
				else if (args[i].isInt()) {
					al.add(new Integer(args[i].getInt()));
				}
				else if (args[i].isFloat()) {
					al.add(new Float(args[i].getFloat()));
				}				
				al.add(new String(" "));
			}
			catch (Exception e) {
				postError(e);
			}
		}
		//post("al:" + al);			
	}
	
	public void bang() {
		// do nothing
	}

	private String getDateTime() {  
	    //datef.setTimeZone(TimeZone.getTimeZone("PST"));  
	    return datef.format(new Date());  
	}
	
	private void postError(Exception e) {
		post("writeText: " + e);
	}
		
}













